
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           KREIRANJE MATRICA I VEKTORA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Direktnim unosom

% vektor redak:
v=[1 2 3]    % ili v=[1,2,3];

% vektor stupac
v1=[1; 2; 3]

% matrica dimenzije 2 x 4
A=[1 5 -1 3; 2 1 3 5]


% kompleksan vektor   

v2=[2;4;1]+i*[1;2;3]   % ovdje je i imaginarna jedinica (i^2=-1)

% kompleksna matrica dimenzija 2 x 2
A1=[i, 1+0.5*i; -2-i, 4-5*i]

% 2) Kreiranje matrice sa slucajnnim elementima  (vidjeti: help rand, help randn )

B=rand(3)

C=rand(3,6)


% 3) KREIRANJE "POSEBNIH" VRSTA MATRICA 

M1=zeros(2,5)          % nul matrica dimanzija 2 x 5

M2=ones(2,3)           % matrica dimenzija 2 x 3 kojoj su svi elementi jednaki 1

M3=diag([1; 2; 3])     % diagonalna matrica

M4=eye(5)              % jedinicna matrica dimenzija 5 x 5

M5=blkdiag(M2,M3,M4)      % blok dijagonalna matrica


% 4) Kreiranje matrice unosom podmatrica

A=diag([1;2;3])
B=rand(3,2);

C=[A, B; zeros(2,size(A,2)), ones(2,size(B,2))]

% kreiranje matrica "for, while,..." petljama

%%% PRIMJER 1 %%%
M=[]      % definiramo M kao paznu matricu.
for i=1:5
    M(i,i)=i
end
% napomena: primjer bi radio i da nismo definirali M=[]; ali ako je M vec ranije definirana (u radnoj ). 
% Alternativno, prije "for" petlje u ovom primjeru mogli smo koristiti "clear M"


%%% PRIMJER 2 %%%
M=eye(3)
for i=1:3
    M(3+i,end)=10*i
end
size(M)


%%% PRIMJER 3 %%%
clear M
for i=1:5
    for j=1:5
        if i<j
            M(i,j)=0
        else
            M(i,j)=1
        end
    end
end


% 5) sparse matrice ("rijetke" matrice: velik broj elemenata u matrici jedak je nuli)

N=sparse(10,4,1)
size(N)
N(2,3)
N(10,4)
N(2,2)=2


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           INDEKSIRANJE, ODABIRANJE ELEMENATA MATRICE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

v=1:5     % kreira vektor-redak v=[1 2 3 4 5]

v1=v'     % vektor-redak  -->   vektor-stupac ( ' oznacava transponiranje ) 

M=[1 2 3; 4 5 6; 7 8 9]

v(3)     % treci element vektora v
v1(4)    % treci element vektora v

M(2,3)   % element matrice M iz drugog retka i treceg stupca

M(6)     % sesti element iz vektora koji se dobije kad se M "razvije" po stupcima

M(:)     % vektor stupac koji sadrzi sve elemenata matrice M "razvijene po stupcima"  

M(2,:)   % drugi redak 
M(:,1)   % prvi stupac

M(2:3,1:2)  % podmatrica matrice M; elementi iz 2. i 3. retka, 1. i 2. stupca 

M([1 3],[1 3])  % podmatrica matrice M; elementi iz 1. i 3. retka, 1. i 3. stupca

M1=M([3 2 1],:)    % matrica M1 dobivena je iz M zamjenom 1. i 3. retka 

M1(:,2)=[]    % brisanje drugog stupca matrice M1


A=[1 2 3; -1 -4 2; -2 4 1]
A>0      % 1 = istina (true); 0 = neistina (false)
A>1
A(A>0)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           OSNOVNE OPERACIJE MATRICNE ALGEBRE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% OSNOVNE ARITMETICKE OPERACIJE

%     A+B      zbrajanje matrica iste dimenzije
%     A-B      oduzimanje
%     A*B      monzenje (matricno)
%     A.*B     mnozenje "element po element"
%     A/B      desno matricno dijeljenje  (A*inv(B))
%     A./B     desno dijeljeneje element po element
%     A\B      lijevo matricno dijeljenje (za kvadratne regularne matrice kao i inv(A)*B)
%     A.\B     lijevo dijeljeneje element po element
%     A^B      potenciranje
%     A.^B     potenciranje element po element
%     A'       konjugirano transponirana matrica
%     A.'      transponirana matrica

A=[1 2 3; 4 5 6]
B=[1 1 1; -1 -1 -1; 1/3 1/3 1/3]'
A*B

A=[1 2 3]; B=[2 2 2]
A./B
A.\B

A=[1+i 2-2*i; -i 4]
A'
A.'

% Primjer: pokazite da je matrica M=[0,2,1; 0,0,-3; 0,0,0] nilpotentna.
M=[0,2,1; 0,0,-3; 0,0,0]
M^2
M^3

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           SKALARNI I VEKTORSKI PRODUKT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% skalarni produkt

x=[1; 2; 3]
y=[3; -2; 4]
dot(x,y)
x'*y

%%% PRIMJER 1 %%%
% provjeri da li su vektori [3; 1; 4] i [-1; 1; -0.5] okomiti
x=[3; 1; 4]
y=[-1; 1; 0.5]
dot(x,y)   % jesu. objasnite?

%%% PRIMJER 2 %%%
% kut izmedju sva vektora
a=[1;1];
b=[1;0];
kut = acos((a'*b)/(norm(a,2)*norm(b,2)))
pi/4


% vektorski produkt

v=[1; 0; 0]
w=[0; 1; 0]
cross(v,w)
cross(w,v)

%%% PRIMJER 3 %%%
% provjerite da li su vektori [3; -2; 5] i [-1.5; 1; -2.5] paralelni
v=[3; -2; 5]
w=[-1.5; 1; -2.5]
cross(v,w)    % jesu. objasnite?


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           SUSTAVI LINEARNIH JEDNADZBI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Rjesavanje sustava A*x=b

%%%  1) A je kvadratna matrica n x n %%%


A=[-2,3,1; 2,2,-1; 1,3,2]
b=[1;2;3];


x=A\b        % rjesenje sustava Ax=b, tj. x=A\b

% provjera:
r=A*x-b

% rjesenje koristeci inverznu matricu: inv(A)
x1=inv(A)*b   % numericki je bolje koristiti A\b nego inv(A)*b

r1=A*x1-b     % uocite da je r1 razlicito od gore izracunatog r 
              % (drugim numeirickim putem su dobiveni rezultati)      


%%% 2) A je matrica n x m gdje je n>m, broj redaka > broj stupaca %%%

% eng. "overdetermined system"
% najcesce jednadzbe nemaju rjesenje (n jednadzbi, m nepoznanica)
% operator \ daje "least square solution", tj. takav vektor x za koji je 
% vrijednost ||b-A*x||, gdje || . || oznacava euklidsku normu vektora (p=2 normu, vidi dolje) 

%%%%%%%
% primjer a)

A=[1 0; 0 1; 0 0]
b=[0.5;0.5;0.5]

% uocite da vektor b ne moze biti prikazan kao linearna kobimbinacija
% vektora a1 i a2, gdje je a1 prvi stupac iz matrice A, a a2 je drugi
% stupac iz matrice A. To znaci da vektor b nije u slici od matrice A, sto
% dalje znaci da jednadzba Ax=b nema rjesenja. Ipak MATLAB ce nam dati
% numericke vijednosti za x kad ovaj sustav pokusamo rijesiti:

x=A\b

b1=A*x         % znamo da Ax nije jednokao b. Izracunajmo koliko je Ax i nazovimo taj vektor sa b1
r=b1-b          % "rezidual": razilika izmedju b i b1. Matlab nam je dao takav x, da je rezidual najmanji moguci

% teorija nam kaze da je r (rezidual) ortogonalan prema stupcima matrice A
% provjera:
r'*A

% graficki prikaz

help plot3     % pogledajte cemu sluzi i kako se koristi naredba plot3

plot3([0,A(1,1)],[0,A(2,1)],[0,A(3,1)],'b','LineWidth',3)    % graficki prikaz vektora prvog stupca matrice A
hold on
plot3([0,A(1,2)],[0,A(2,2)],[0,A(3,2)],'b','LineWidth',3) % graficki prikaz vektora drugog stupca matrice A
plot3([0,b(1)],[0,b(2)],[0,b(3)],'r','LineWidth',3)  % graficki prikaz vektora b
plot3([0,b1(1)],[0,b1(2)],[0,b1(3)],'g','LineWidth',3)  % graficki prikaz vektora b1=Ax, kad je x najbolje priblizno rjesenje sustava Ax=b
plot3([b(1),b(1)+r(1)],[b(2),b(2)+r(2)],[b(3),b(3)+r(3)],'k','LineWidth',3) % gradicki prikaz reziduala (r=b1-b)
xlabel('x')
ylabel('y')
zlabel('z')
grid on
axis equal

%%%%%%%%%
% primjer b)  "overdetermined system"

a1=[2;0;1];
a2=[-1; 2; 2];
a3=a1+a2;
a4=0.5*a1+1.2*a2;


A=[a1, a2, a3, a4]
b=ones(3,1)

x=A\b

b1=A*x
r=b1-b

% teorija nam kaze da je r (rezidual) ortogonalan prema stupcima matrice A
% provjera:
r'*A

% graficki prikaz

help plot3

plot3([0,A(1,1)],[0,A(2,1)],[0,A(3,1)],'b','LineWidth',3)
hold on
plot3([0,A(1,2)],[0,A(2,2)],[0,A(3,2)],'b','LineWidth',3)
plot3([0,A(1,3)],[0,A(2,3)],[0,A(3,3)],'b','LineWidth',3)
plot3([0,A(1,4)],[0,A(2,4)],[0,A(3,4)],'b','LineWidth',3)

plot3([0,b(1)],[0,b(2)],[0,b(3)],'r','LineWidth',3)
plot3([0,b1(1)],[0,b1(2)],[0,b1(3)],'g','LineWidth',3)
plot3([b(1),b(1)+r(1)],[b(2),b(2)+r(2)],[b(3),b(3)+r(3)],'k','LineWidth',3)
xlabel('x')
ylabel('y')
zlabel('z')
grid on
axis square


%%%%%%%%%%%%%%%%%%%%%%%%


%%% 3) A je matrica n x m gdje je n<m, broj redaka < broj stupaca

% eng. "underdetermined system"
% u ovom slucaju broj nepoznanica je veci nego broj jednandzbi
% MATLAB ce dati jedno ("partikularno") rjesenje
% skup svih rjesenja dobiva se linearnom kombinacijom patikularnog rjesenja
% i vektora iz baze prostora jezgre matrice (eng. "kernel" space ili "null" space)

clear all

A=[2 -1 4 2; 1 2 3 4]
b=[1; 2]

x=A\b    % partikularno rjesenje

Z=null(A)  % jezgra matrice A; dimezija matrice Z u ovom slucaju je 4 x 2
% za svaki stupac iz Z, vrijedi da je A*(stupac iz Z)=0 (vektor sa nulama)

A*Z(:,1)
A*Z(:,2)

% svaka linearna kombinacija partikularnog rjesenja x 
% i stupaca od Z je rjesenje pocetne jednadzbe
% PROVJERA

A*(x+2*Z(:,1)+2*Z(:,2))   % =b
b


%%% PRIMJER: "overdetermined system" %%%%%%%%%%%%%%%%%%
% ZADATAK: potrebno je pronaci jednadzbu pravca koji 
% prolazi kroz sljedece 4 tocke u ravnini:
% (x1,y1)=(-6 5.5)
% (x2,y2)=(-4 3)
% (x3,y3)=(0 1.8)
% (x4,y4)=(6 -0.5)

% jednadzba pravca: y = a*x + b  , gdje su a i b trazene konstante
% 
%  xi*a+b=yi,   i =1,...,4
%
% u matricnom obliku: X*c=y  

X=[-6 1; -4 1; 0 1; 6 1]
y=[5.5; 3; 1.8; -0.5]

ab=X\y;
a=ab(1)
b=ab(2)

x=-10:10;
y=a*x+b;
plot(x,y)
hold on
plot([-6 -4 0 6],[5.5 3 1.8 -0.5],'*r')
grid on
%%% Kraj primjea %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% FUNKCIJA rref

% rref(A) daje po redcima reduciranu echelonsku
% formu od matrice A
% vise detalja: help rref

A=round(5-10*rand(3,5))
A1=rref(A)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           RANG MATRICE, PROSTOR SLIKE I JEZGRE MATRICE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A=[1 2 3; 3 2 1; 4 4 4]

rank(A)    % rang matrice
orth(A)    % slika matrice, "image space", prostor stupaca
null(A)    % jezgra, nul-prostor, "null space", "kernel space"
orth(A')'  % prostor redaka, "row space" 
null(A')'  % lijevi nul-prostor, "left null space"

% vektori dobiveni sa null i orth su ortonormalni. Kreirani su koristenjem
% dekomposicije singularne vrijednosti

%%% PRIMJER 1 %%%
A=randn(2,3)   % vidjeti: help randn
A=A>=0
A=double(A)

slika=orth(A) 
jezgra=null(A)

A*jezgra

%%% PRIMJER 2: rang matrice %%% 
v1=rand(3,1);
v2=rand(1,4);
V=v1*v2
rank(V)

v1=rand(3,2);
v2=rand(2,4);
V=v1*v2
rank(V)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           DETERMINANTA I TRAG MATRICE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% determinanta matrice

A=[-1 1; 2 5]
det(A)

A=diag([1;2;3])
det(A)        % Detarminanta dijagonalne matrice je jednaka umnosku elemenata na dijagonali 

A=[1 2 3; 3 2 1; 4 4 4]
det(A)        % Determinanta je nula: matrica je singularna ( 4. redak je zbroj prva dva )

% trag matrice: zbroj elemenata na dijgonali matrice

A=[1 2 3; 4 5 6; 7 8 9]
trace(A)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           VEKTORSKE I MATRICNE NORME
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%     For matrices...
%       NORM(X) is the 2-norm of X.
%       NORM(X,2) is the same as NORM(X).
%       NORM(X,1) is the 1-norm of X.
%       NORM(X,inf) is the infinity norm of X.
%       NORM(X,'fro') is the Frobenius norm of X.
%       NORM(X,P) is available for matrix X only if P is 1, 2, inf or 'fro'.
%  
%     For vectors...
%       NORM(V,P) = sum(abs(V).^P)^(1/P).
%       NORM(V) = norm(V,2).
%       NORM(V,inf) = max(abs(V)).
%       NORM(V,-inf) = min(abs(V)).


%%% PRIMJER 1 %%%

v=[1;1]
norm(v,2)  % ili norm(v,2) = euklidska norma
sqrt(2)

%%% PRIMJER 2 %%%
% Treba odrediti jedinicni vektor (vektor duljine 1 u euklidskoj normi)
% paralelan sa vektorom ("u smjeru vektora") [1; 2; 4]

v=[1; 2; 4]
vJedinicni=v/norm(v,2)
norm(vJedinicni,2)

%%% PRIMJER 3 %%%
A=randn(5,5)
norm(A,2)  
svd(A)           % svd daje singularne vrijednosti matrice (vidjeti dolje)     
sqrt(eig(A'*A))  % eig deje svojstvene vrijednosti matrice (vidjeti dolje)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           SVOJSTVENE VRIJEDNOSTI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% vidjeti: help eig

help eig

%%%%%%%%%%%%%%
%%%%  PRIMJER A

M=randn(5,5)
[V,D]=eig(M)

a=M*V(:,1)
b=D(1,1)*V(:,1)

% M*V=V*D
% M=V*D*inv(V)      dekompozicija svojstvene vrijednosti

V*D*inv(V)
M

%%%%%%%%%%%%%%%%%%
%%% PRIMJER B: "eigshow" = interaktivni primjer za srtanje svojstvenih vektora (i singularnih vektora)

%%%% NAPOMENA:
% U statijim verzijama MATLABA ova funkcija je ugradjena u osnovni dio
% matlaba
%
% Za novije verzije MATLABA za ovu fonkciju potrebno je instalirati 
% toolbox "Numerical Computing with MATLAB "
% koji se nalazi na sljedecoj adresi (za skidanje potrebno imati matlab account)
%   https://www.mathworks.com/matlabcentral/fileexchange/37976-numerical-computing-with-matlab
%%%%%%%%%%%%%

% vidjeti upute u helpu
help eigshow

% pokretanje primjera
eigshow


%%%%%%%%%%%%%%%%%%
%%% PRIMJER C: svojstvene vrijednost realne simetricne matrice su realni brojevi
%%% svojstveni vektori simetricne matrice su ortogonalni

N=rand(5,5);
N=N'+N

[V,D]=eig(N); 
D             % svojstvene vrijednosti su realni brojevi

V'*V          % svojstveni vektoru su medjusobno okomiti

%%%%%%%%%%%%%%%%%
%%% PRIMJER D: svojstvene vrijednost pozitivno (semi)definitne matrice su nenegativne
%%% svojstveni vektori 

N=[1,4,-2;3,2,5;-2,-4,2];
N=N'*N
eig(N)


%%%%%%%%%%%%%%%%
%%% PRIMJER E: svojstvene vrijednosti i karakteristican polinom

N=[1,4,-2;3,2,5;-2,-4,2];

CarPol=poly(N)   % definira karakteristicni polinom matrice N (vidjeti: help poly)

roots(CarPol)    % nultocke karakteristicnog polinoma
eig(N)           % svojstvene vrijednosti 


%%%%%%%%%%%%%%%%%
%%% PRIMJER F: Caley-Hamilton teorem:
% svaka matrica zadovoljava svoju karakteristicnu jednandzbu

N=[1,4,-2;3,2,5;-2,-4,2];

CarPol=poly(N)   % definira karakteristicni polinom matrice N (vidjeti: help poly)

CarPol(1)*N^3+CarPol(2)*N^2+CarPol(3)*N+CarPol(4)*eye(3)

%%%%%%%%%%%%%%%
%%%%  PRIMJER G: stabilnosti i ponasanje dinamickog sustava (linearne diferencijalne jednadzbe)

% NAPOMENA: u ovom primjeru rjesavamo diferecijalne jednadzbe koristeci
% funciju ode23. Detalje o rjesavanju dif. jednadzbi radit cemo na kasnijim
% predavanjima. Naglasak u ovom primjeru je veza izmedju svojstvenih
% vrijednosti i ponasanja dinamickog sustva

% diferecijalan jednadba zadaba je oblikom 
% xd/dt =  Adif * x
% gdje je Adif matrica. Ova matrica ovdje je definirana kao globalna
% varijable jer se koristi u funciji "DifJednadba".

global Adif

%%%%

Adif=[1 20;-20 1];

eig(Adif) % realni djelovi svojstevnih vrijednosti su pozitivni -> sustav je nestabilan


x0=[4;0]   % Pocetni uvjet

[t,x] = ode23(@DifJednadzba,[0 4],x0);
figure(1)
plot(t,x(:,1),'r')
hold on
plot(t,x(:,2),'b')

%%%%

Adif=[-1 20;-20 -1];

eig(Adif) % realni djelovi svojstevnih vrijednosti su negativni -> sustav je stabilan


x0=[4;0]   % Pocetni uvjet

[t,x] = ode23(@DifJednadzba,[0 4],x0);
figure(1)
plot(t,x(:,1),'r')
hold on
plot(t,x(:,2),'b')

%%%%

Adif=[0 20;-20 0];

eig(Adif) % realni djelovi svojstevnih vrijednosti su nula -> sustav nije asimptotski stabilan


x0=[4;0]   % Pocetni uvjet

[t,x] = ode23(@DifJednadzba,[0 4],x0);
figure(1)
plot(t,x(:,1),'r')
hold on
plot(t,x(:,2),'b')

%%%%

NN=10;
Adif=-5*eye(NN)+rand(NN,NN);

eig(Adif)   % pogledajte svojestve vrijednosti i zakljucite je li sustav stabilan ili nije

x0=rand(NN,1)   % Pocetni uvjet

[t,x] = ode23(@DifJednadzba,[0 40],x0);
figure(1)
plot(t,x)



%%%%%%%%%%%%%%%%%%
%%%%  PRIMJER H: glavna naprezanja 2D

% Zadan je tenzor naprezanja

sigma=[1,-1/2; -1/2, 2];

% potrebno je odrediti vrijednosti i sjmerove glavnih naprezanja

[Smjerovi,Naprezanja]=eig(sigma)

% napomena: smjerovi (svojstveni vektori) glavnih nparezanja su stupci
% matrice "Smjerovi". Ovi vektori imaju jedinicnu duljinu (u drogoj - Euklidskoj - normi)

GlavnaOs1=Naprezanja(1,1)*Smjerovi(:,1);    % glavna os naprezanje (vektor u smjeru glavne osi, duljine jednake vrijednosti glavnog naprezanja)
GlavnaOs2=Naprezanja(2,2)*Smjerovi(:,2);    % glavna os naprezanje (vektor u smjeru glavne osi, duljine jednake vrijednosti glavnog naprezanja)
figure(200)
plot([GlavnaOs1(1),-GlavnaOs1(1)],[GlavnaOs1(2),-GlavnaOs1(2)],'r','LineWidth',3)
hold on
plot([GlavnaOs2(1),-GlavnaOs2(1)],[GlavnaOs2(2),-GlavnaOs2(2)],'b','LineWidth',3)
grid on
axis equal

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           SINGULARNE VRIJEDNOSTI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% "Djelovanje" matrica: rotacija i produljenje/smanjenje

% PRIMJER A

A = [0 1; -1 0];   % ova matrica A kod umnoska Ax rotira vektor x za 90 stupnjeva
x = [1; 1];
y = A*x;
plot([0 x(1)],[0 x(2)],'b','LineWidth',3);
hold on
plot([0 y(1)],[0 y(2)],'r','LineWidth',3);
grid on
axis equal

% PRIMJER B

A = [1 2; -1 0];    % kod ove matrice, uz rotaciju dobili smo i produljenje vektora ("izlazni "vektor y=Ax je dulji od "ulaznog" x)
x = [0; 1];
y = A*x;
plot([0 x(1)],[0 x(2)],'b','LineWidth',3);
hold on
plot([0 y(1)],[0 y(2)],'r','LineWidth',3);
grid on
axis equal

% PRIMJER C

A = [1 2; -1 0];    % kod ove matrice, uz rotaciju dobili smo i produljenje vektora ("izlazni "vektor y=Ax je dulji od "ulaznog" x)

figure(100)
hold on

N=100;
for k=1:N;
    kut=2*pi/N;
    x = [cos((k-1)*kut); sin((k-1)*kut)];
    y = A*x;
    plot([0 x(1)],[0 x(2)],'b','LineWidth',2);
    hold on
    plot([0 y(1)],[0 y(2)],'r','LineWidth',2);
end
grid on
axis equal

%%% PRIMJER D : interaktivni primjer za srtanje svojstvenih vektora (i singularnih vektora)

% vidjeti upute u helpu:
help eigshow

% pokretanje primjera
eigshow


% SINGULARNE VRIJEDNOSTI
%
% help svd
%
% s = svd(X) returns a vector of singular values.
% 
% [U,S,V] = svd(X) produces a diagonal matrix S of the same dimension as X, 
% with nonnegative diagonal elements in decreasing order, 
% and unitary matrices U and V so that X = U*S*V'.

M=[1 2; 3 4; 5 6; 7 8]

[U,S,V]=svd(M)

% U i V su unitarne matrice:

U'*U
V'*V

% interpretacija SVD dekompozicije
Ulaz1=V(:,1);
Izlaz1=U(:,1);
Pojacanje1=S(1,1);

M*Ulaz1
Pojacanje1*Izlaz1




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           JOS PRIMJERA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% PRIMJER: Linearna algebra u grafici (rotacije, translacije, refleksije, perspektiva)
% i robotici

% vidjeti funkciju rot2d

x = [1 2 2 1]; 
y = [0 0 1 1];

[xt, yt] = rot2d(90, x, y)


